/********************************************************************/
/*                                                                  */
/*		Pulswellen Detektor											*/
/*                                                                  */
/*		(C) Tilo Lutz, TiloLutz@gmx.de								*/
/*																	*/
/*		02.11.2006 Ansteuerung DAC0									*/
/*																	*/
/*		Lse mit Timer2 alle 1ms einen Interrupt aus				*/
/*		Verwende Timer3 fr Verzgerungen							*/
/********************************************************************/


#include <ADuC7020.h>

// Eigene Programmteile
#include "main.h"
#include "uart.c"
#include "lib.c"

void Init (void) {
   	IRQ = My_IRQ_Function;	// Specify Interrupt Service Routine

	// DAC0 Einstellungen
	LED = 2000; // Setze Startwert LED Spannung
	Offset = 2024;
	// P4.3 wird zum Steuern des S&H Glied verwendet
	GP4DAT |= 0x0C000000; // P4.3 ist Sample&Hold Ausgang, P4.2 ist LED
	GP4SET = 0x040000; // Schalte LED aus, LED ist invertiert.

	// Timer2 Einstellungen
	T2CON = 0x04C0; // Periodischer Timer, f=32kHz
	T2LD = 0x1F; // 32kHz / 32 = 1kHz = 0x20 Timer lst alle 1ms aus.

	// Setze Startwerte
	tempdatabufferready=0;
	readpositiondatabuffer=0;
	writepositiondatabuffer=0;
	PulseWait=0;
	ControlWait=2000; // Regele Offset alle 10sek nach.
	p2pdistance=0;
	p2pdistancespos=0;
	for (int i=0; i<sizeofp2pdistances; i++) {
		p2pdistances[i][0] = 0;
		p2pdistances[i][1] = 0;
	}


	// Aktiviere ADC0
	//REFCON = 0x01; // Aktiviere interne Referenz
	ADCCON = 0x07A3; // Single Ended Mode, Single conversation, power up

	IRQEN = IRQEN | WAKEUP_TIMER_BIT; // Enable Timer1 IRQ

	// Suche Arbeitspunkt
	GainControl();
	OffsetControl();
}

int GetADC(int Port) {
	while (ADCCON & 0x40) {} // Warte bis ADC frei ist
	ADCCP = Port;
	ADCCON = 0x07A3; // Single Ended Mode, Single conversation, power up
	delay(5);
	while (!ADCSTA) {} // Warte bis AD Wandlung fertig ist.
	return (ADCDAT >> 16);
}

void OffsetControl(void) {
	if (ADCBuffer[2]<2020 || ADCBuffer[2]>2070) { // Regele Offsetabgleich V=100 -> deltaUMax=1,65V/V/4 == 4,125m == 5,12 2048+-(5,12/2)
		WritelnUart("OffsetControl");
		Offset=0;
		for (int i=11; i>0; i--) {
			Offset += (1 << i);
			// Warte bis LED Pegel gendert wurde und System eingeschwungen ist
			sleep(200-i*i);
			if (ADCBuffer[2]<2048) Offset -= (1 << i); // Reduziere Spannung, wenn Spannung zu hoch.
				// Spannung noch zu niedrig. Wird aber im nchsten Schleifendurchlauf hher.
			}
		}
	// Feinabgleich Offset direkt am Ausgang
	if (ADCBuffer[2]<2048) { // Setze letztes Bit
		Offset++;
	}
	sleep(100);
	// ndere Offset in 10er-Schritten, bis Ausgang springt.
	// ndere dann Offset in 1er Schritten bis Mitte erreicht
	// Halte Signal mind. 2sek in der Mitte
	int abort=10; // Versuche Abgleich 10sek lang
	int LastSample = ADCBuffer[0];
	while (abort>0) {
		if (ADCBuffer[0]>4000 && LastSample>4000 && Offset<4085) Offset +=10; // Obere Grenze
		else if (ADCBuffer[0]<100 && LastSample<100 && Offset>10) Offset -=10; // Untere Grenze
		else abort=0; // Signal ist gesprungen oder liegt nicht am Rand
		LastSample = ADCBuffer[0];
		sleep(200);
		abort--;
		}
	// Signal sollte jetzt irgend wo in der Mitte liegen
	abort=10;
	int sum=0;
	while (abort>0) {
		// Bilde Mittelwert ber die Messwerte der letzten 100ms
		for (int i=0; i<100; i++) {
			sum += ADCBuffer[0];
			sleep(1);
			}
		sum /= 100;
		if (sum<1500 && Offset>1) Offset--;
		if (sum>2500 && Offset<4094) Offset++;
		sleep(100);
		abort--;
		}
	// Signal sollte jetzt genau in der Mitte liegen
	if (ADCBuffer[0]<807 || ADCBuffer[0]>3289) // Gleiche Sendeleistung an, wenn Mebereich nicht erreicht werden konnte.
		GainControl();
}

void GainControl(void) {
	// Automatische Leistungskontrolle der LED
	if (ADCBuffer[1]<2730 || ADCBuffer[1]>3848) {
		WritelnUart("GainControl");
		// Nhere auf 2,65V(3289) in 12 Schritten an.
		LED=0;
		for (int i=11; i>0; i--) {
			LED += (1 << i);
			// Warte bis LED Pegel gendert wurde
			sleep(10);
			if (ADCBuffer[1]>3289) LED -= (1 << i); // Reduziere Spannung, wenn Spannung zu hoch.
				// Spannung noch zu niedrig. Wird aber im nchsten Schleifendurchlauf hher.
		}
		sleep(10);
		if (ADCBuffer[1]>3289) LED--; // Setze letztes Bit
	}
	
}

int main (void)  {
	InitUart(9600);
	Init();
	WritelnUart("Pulsmesser bereit.");
	while (1) { // Endlosschleife fr Hauptprogramm
		GainControl();
		 if (ControlWait==0) { // Langsame Regleung Offset, immer Aktiv
			// Bilde Mittelwert aller Messdaten im Speicher.
			int sum=0;
			for (int i=0; i<sizeofdatabuffer; i++)
				sum += databuffer[i];
			sum /= sizeofdatabuffer;
			if (sum<100 || sum>4000) { // Signal liegt schon zu lange an Anschlag -> Messbereich fr zu lange Zeit verlassen -> Fhre Grobabgleich durch
				OffsetControl();
			}
			else if (sum<1948 || sum>2148) { // Signal soll in der Mitte +-1V liegen.
				WritelnUart("OffsetFineControl");
				if (sum<2048 && Offset>1) Offset--;
				if (sum>2048 && Offset<4094) Offset++;
			}
			ControlWait=1000; // Regele Offset jede sek nach.
		}
	
		if (writepositiondatabuffer != readpositiondatabuffer) { // bertrage Daten aus Puffer, wenn neue Daten vorhanden
			//int oldposition = readpositiondatabuffer % sizeofdatabuffer;
			readpositiondatabuffer++;
			readpositiondatabuffer = readpositiondatabuffer % sizeofdatabuffer;

			/* Eigenschaften einer Pulswelle:
			 * negative Krmmung
			 * Spitze Steigung ca. 0
			 * Steigungsangaben entsprechen meinem Ruhepuls bei Ta=50ms
			 * Mind ber 
			*/
			
			// Bestimme Punkte
			int peakposition;
			if (readpositiondatabuffer>4) peakposition=readpositiondatabuffer-5;
				else peakposition=sizeofdatabuffer+readpositiondatabuffer-5;
			int prepeakposition;
			if (readpositiondatabuffer>5) prepeakposition=readpositiondatabuffer-6;
				else prepeakposition=sizeofdatabuffer+readpositiondatabuffer-6;
			int riseposition;
			if (readpositiondatabuffer>14) riseposition=readpositiondatabuffer-15;
				else riseposition=sizeofdatabuffer+readpositiondatabuffer-15;
			int rise = databuffer[peakposition]-databuffer[riseposition];
			int fall = databuffer[readpositiondatabuffer]-databuffer[peakposition];
			int peak = databuffer[peakposition]-databuffer[prepeakposition];
			int krumm = fall-rise;
			// berprfe, ob es sich umn Spitze handelt
			if (PulseWait==0) {
				if (rise>500 && rise<1000) {
					if (peak<70 && peak>-70) {
						if (fall<-50 && fall>-500) {
							if (krumm<-500 && krumm>-1000) {
								PulseWait=300; // Refraktrzeit ca. 300ms
								p2pdistancespos++;
								p2pdistancespos %= sizeofp2pdistances;
								p2pdistances[p2pdistancespos][0] = p2pdistance;
								p2pdistances[p2pdistancespos][1] = databuffer[readpositiondatabuffer];
								p2pdistance=0;
								int sum=0;
								int dist=0;
								for (int i=p2pdistancespos; i!=p2pdistancespos+1; i--) { // Ergibt immer Lsung und bricht immer ab
									if (i<0) i=sizeofp2pdistances-1;
									if (p2pdistances[i][0]!=0) {
										dist += p2pdistances[i][0];
										sum++;
										if (dist>PulseTime) break; // Breche Ab, wenn Zeit erreicht
									}
								}
								int puls = 3000 * sum; // 60.000ms / 20ms = 3000
								puls /= dist;
								WriteUart("Puls: ");
								IntToUart(puls);
							}
							else WritelnUart("Abfall");
						}
						else WritelnUart("Spitze");
					}
					else WritelnUart("Anstieg");
				}
			}
		}
	}
}

void My_IRQ_Function (void) {				
	if ((IRQSTA & UART_BIT) != 0) { // Uart
		DAC3DAT = (4000 << 16); // Setze Pegel DAC0
		DAC3CON = 0x13; // Aktiviere DAC0 (Sende-LED)
		//WritelnUart("XXXXXXXXXXX");
		IrqUart();
	}
	if ((IRQSTA & WAKEUP_TIMER_BIT) != 0) { // Timer2?

		// Speichere Abtastwerte in Register bevor LED Strungen verursacht.
		ADCBuffer[0] = 4095 - GetADC(0);
		ADCBuffer[1] = GetADC(1);
		ADCBuffer[2] = GetADC(2);
		udelay(30); // Warte bis Strspitze vorbei.

		if (ControlWait>0) ControlWait--;
		if (PulseWait>0) PulseWait--;
		if (sleeptimer>0) sleeptimer--;

		DAC0DAT = (LED << 16); // Setze Pegel DAC0
		DAC0CON = 0x13; // Aktiviere DAC0 (Sende-LED)
		DAC1DAT = (Offset << 16); // Setze Pegel DAC1
		DAC1CON = 0x13; // Aktiviere DAC1 (Offset-Abgleich)
		udelay(25);
		GP4SET = 0x080000;
		/* Warte ca. 30us, bis Spitze am S&H Glied anliegen
		*  Schalte Triggeranschlu ein
		*  Warte 30us bis S&H Kondensator geladen ist.
		*  Schalte Triggeranschlu ab.
		*  Lese Pegel an ADC0 ein.
		*  Schalte LED ab
		*/
		udelay(30); // Warte bis Kondensator geladen
		GP4CLR =  0x080000; // Deaktiviere Sample und HOLD
		udelay(60); // warte bis MOSFET sperrt.
		DAC0DAT = 0x00; // Deaktiviere DAC0

		tempdatabuffer[tempdatabufferready] = ADCBuffer[0]; // Lese Wert und invertiere
		tempdatabufferready++;
		if (tempdatabufferready==sizeoftempdatabuffer) {
			writepositiondatabuffer++;
			writepositiondatabuffer = writepositiondatabuffer % sizeofdatabuffer;
			int sum=0;
			for (int i=0; i<sizeoftempdatabuffer; i++)
				sum += tempdatabuffer[i];
			sum /= sizeoftempdatabuffer;
			DAC2DAT = (sum << 16); // Setze Pegel DAC2
			DAC2CON = 0x13; // Aktiviere DAC2 Ausgabe der Messdaten
			databuffer[writepositiondatabuffer] = sum; // Schreibe Wert
			tempdatabufferready=0;
			p2pdistance++; // Erhhe Abstandsmesser
			T2CLRI = 0;	// Clear Timer IRQ
			}
	}
	return ;
} 

void IntToUart(int integer) { // Sende Messwert
	char stringint[12];
	IntToStr(stringint, integer); // Wandle Messwert in String um.
	WritelnUart(stringint);
}


void sleep(int time) {
	sleeptimer=time;
	while (sleeptimer>0) {}
}
