/********************************************************************/
/*                                                                  */
/*		Uart Funktionen zum Senden und Empfangen von Textfolgen		*/
/*		ber den uart.                                              */
/*                                                                  */
/*		(C) Tilo Lutz, TiloLutz@gmx.de								*/
/*																	*/
/*		03.11.2006	Basisfunktionen von Marcus Moser bernommen.    */
/*					Gesammte funktionen berarbeitet und ergnzt.	*/
/*																	*/
/********************************************************************/

#include "ascii.h"
#include "uart.h"

char GetCharUart (void) { // Lese char aus uart
   	while ((COMSTA0 & 0x01)==0) {} // Warte bis Daten an uart anliegen
  	return (COMRX); // Gebe Buffer aus
}

/*
int PutCharUart (char ch) {	// Schreibe Char auf uart
   	while ((COMSTA0 & 0x40)==0) {} // Warte bis Sendepuffer leer ist
	if (ch==LF) { // Wandle Zeilenende um
		COMTX = CR; // Sende Zeichen
		return 0;
	}
	COMTX = ch; // Sende Zeichen
	return 0;
}
*/

int PutCharUart (char ch) {
	if (ch==LF) ch=CR; // Wandele Zeilenende um.
	if ((COMSTA0 & 0x40)==1) { // Sendepuffer leer, Sende nchstes Zeichen
		COMTX = ch;
	}
	else { // Kopiere Wert in Puffer
		writeuarttxbuffer++;
		writeuarttxbuffer %= sizeofuartbuffer;
		uarttxbuffer[writeuarttxbuffer] = ch;
	}
	return 0;
}


void IrqUart (void) {
	if (writeuarttxbuffer!=readuarttxbuffer) { // Zeichen im Sendepuffer
		if ((COMSTA0 & 0x40)==1) { // Sendepuffer leer, Sende nchstes Zeichen
			readuarttxbuffer++;
			readuarttxbuffer %= sizeofuartbuffer;
			COMTX = uarttxbuffer[readuarttxbuffer]; // low byte
		}
	}
}

int PutIntUart (int ch) {	// Schreibe Char auf uart
   	while ((COMSTA0 & 0x40)==0) {} // Warte bis Sendepuffer leer ist
	COMTX = ch; // Sende Zeichen
	return 0;
}

char WriteUart (char *ptr) { // Schreibt String auf uart
	// *** fixme: mache Funktion binrsicher?
	char *origptr = ptr;
	while (*ptr!='\0') {
		PutCharUart (*ptr); // Sende Zeichen
		ptr++;
	}
	return *origptr;
}

char WritelnUart (char *ptr) { // Schreibt String. String wird mit \n abgeschlossen.
	char *origptr = ptr;
	while (*ptr!='\0') {
		PutCharUart (*ptr); // Sende Zeichen
		ptr++;
	}
	PutCharUart('\n');
	return *origptr;
}

char ReadlnUart (char *buffer, int len) { // Lese bis Zeilenende oder Pufferlnge erreicht.
	char *origbuffer = buffer;
	int position = 0; // Position im Puffer
	do {
		char newchar = GetCharUart();
		if (newchar==CR) {
			*buffer = 0;
			return *origbuffer;
		}
		if (newchar==LF) {
			*buffer = 0;
			return *origbuffer;
			}
		*buffer = newchar;
		buffer++;
		position++;
	} while (position<len);
	buffer[position+1] = 0;
	return *origbuffer;
}

char ReadUart (char *buffer, int len) { // Lese bis Pufferlnge erreicht.
	char *origbuffer = buffer;
	int position = 0; // Position im Puffer
	do {
		char newchar = GetCharUart();
		*buffer = newchar;
		buffer++;
		position++;
	} while (position<len);
	buffer[position+1] = 0;
	return *origbuffer;
}

void InitUart (int rate) { // Initialisiert den Uart fr 9600 baud
	GP1CON |= 0x0011; // Aktiviere uart
	COMCON0 = 0x080; // Erlaube setzen der Baudrate
	int cd = (POWCON & 0x07); // aktuelle Taktrate
	// Formel zur Baudratenbestimmung aus uC Handbuch S. 62
	int dl = (1305625 >> cd) / rate;
	COMDIV0 = (dl & 0x00FF); // Setze Lowbyte
	COMDIV1 = (dl & 0xFF00); // Setze Highbyte
	COMCON0 = 0x003; // Verwende 8bit Wrter mit 1 Stopbit
	// Setze Variablen auf 0, Startbedingung
	readuarttxbuffer=0;
	readuartrxbuffer=0;
	writeuarttxbuffer=0;
	writeuartrxbuffer=0;
	COMIEN0 = 0x03; // Aktiviere Interrupt, wenn RX oder TX Puffer voll
	IRQEN = IRQEN | UART_BIT; // Enable UART IRQ
	return;
}

void StopUart (void) { // Deaktiviert den uart
	GP1CON ^= 0x0011; // Aktiviere uart
	COMCON0 = 0x080; // Erlaube setzen der Baudrate
	COMDIV0 = 0; // Setze Baudrate auf 0
	COMDIV1 = 0; // Setze Baudrate auf 0
	COMCON0 = 0; // Reset Uart Einstellungen
	COMIEN0 = 0; // Schalte Interrupt aus
	//IRQEN = IRQEN & !UART_BIT; // Enable UART IRQ
	return;
}

